#include <mega128.h>
#include <delay.h>
#include <stdio.h>
#include "types.h"
#include "cpu.h"
#include "fat32.h"
#include <string.h>


u08 eeprom *ptrThresholds = 0x0010;
u16 threshold[6]; 
u16 adcval[6];
 
u16 readADC(u08 channel);
void setFieldSensor(u08 channel, u08 shield);
PARTITION partition;

u16 msCount=0;

u08 uart_getc(void)
{
    while (!(UCSR1A & (1<<RXC)));
    return UDR1;                     
}

void setThresholds()
{
    u08 i;
    u08 eeprom *eepPtr = ptrThresholds;
    for (i=0;i<6;i++)
    {
        threshold[i] = eepReadWord(eepPtr);
        eepPtr += 2;
    }    
}

// USART1 RX interrupt
interrupt [USART1_RXC] void usart0_rxc_isr(void)
{
    u08 i;
    u08 c;
    u08 temp;
    u08 eeprom *eepPtr = ptrThresholds;
    
    c = UDR1;
    
    printf("passed: %c",c);
    PORTC ^= 0xFF;
    if (c=='V')
    {
         printf("#%04i%04i%04i%04i%04i%04i#",adcval[0],adcval[1],adcval[2],adcval[3],adcval[4],adcval[5]);  
    }
    
    if (c=='S')
    {
        for (i=0;i<12;i++)
        {
            temp = uart_getc();
            eepWriteByte(eepPtr++, temp);
            printf("passed");          
        }
        setThresholds();
        
    }
}
  

interrupt [TIM1_OVF] void timer1_overflow(void)
{
    msCount++;
}


u16 readADC(u08 channel)
{
  u08 i;
  u16 res;
 
  ADMUX = channel;                      // Kanal einstellen                      
  ADMUX |= (1<<REFS1) | (1<<REFS0);     // Referenzspannung (2,56V intern)
  
  ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung starten um ADC "warmlaufen" zu lassen, Wert wird ignoriert 
  while ( ADCSRA & (1<<ADSC) );         // warten bis fertig
  res = ADCW;                        // Wert auslesen, Dummy-Wandlung beendet
   
  // eigentliche Messung starten, 4 Messungen und daraus der Mittelwert
  res = 0; 
  for( i=0; i<4; i++ )
  {
    ADCSRA |= (1<<ADSC);            
    while ( ADCSRA & (1<<ADSC) );   
    res += ADCW;
  } 
  res /= 4;   
  
  return res;
}


void setFieldSensor(u08 channel, u08 shield)
{
    PORTA = channel;
    if (shield == true)
        PORTA |= (1<<PA3);
    else 
        PORTA &= ~(1<<PA3);
}


void main(void)
{
    u16 i;
    u16 count = 512;
    u32 offset = 0;
    FAT_FILE file;
    FAT_FILE file2;
    char strFile[] = "test.txt";
    char strFile2[] = "test23.txt";
    char buffer[512];
    
    cpu_InitIOPorts();
    cpu_InitUART();
    cpu_InitTimer();
    cpu_InitComp();
    cpu_InitSPI();
    //cpu_EnableInterrupts();
    
    
    printf("\r\nInit Speicherkarte Fat System...\r\n");
    i = fat_init(); 
    if (i != 0)
    {
        switch (i)
        {
            case 1:    printf("\r\nSD Karte nicht gefunden!\r\n");break;         
            case 2:    printf("\r\nFalsches FAT System!\r\n");break;         
            case 3:    printf("\r\nInit fehlgeschlagen!\r\n");break;
        }             
    }
    else
    {
        printf("\r\nSuche Datei '%s'...\r\n",strFile);
        if ( fat_open(strFile,&file) == 0)
        {
            msCount = 0;
            TCNT1=0x00;
            TCCR1B=0x02;
            fat_read_bytes(&file,offset,count,buffer);       
            TCCR1B=0x00; 
            printf("\r\nDateiinhalt (Offset 0x%X%X, %i Bytes):\r\n--------------------------------------\r\n",offset >> 16, offset, count);
           
           
            for (i=0; i< count; i++)
            {           
                printf("%c",buffer[i]);        
            }
            printf("\r\nZeit: %i %X ms\r\n",msCount, TCNT1);  
        }
        else
        {
            printf("\r\nDatei nicht gefunden!\r\n",);
        }        
    
        printf("\r\nSuche Datei '%s'...\r\n",strFile2);
        if ( fat_open(strFile2,&file2) == 0)
        {
            count = file2.file_size;
            offset = 0;
            msCount = 0;
            TCNT1=0x00;
            TCCR1B=0x02; 
            fat_read_bytes(&file2,offset,count,buffer);       
            TCCR1B=0x00; 
            printf("\r\nDateiinhalt (Offset 0x%X%X, %i Bytes):\r\n--------------------------------------\r\n",offset >> 16, offset, count);
            for (i=0; i< count; i++)
            {           
                printf("%c",buffer[i]);        
            } 
            printf("\r\nZeit: %i %X ms\r\n",msCount, TCNT1);  
        }
        else
        {
            printf("\r\nDatei nicht gefunden!\r\n",);
        }        
   
    }
    
    
    cpu_InitADC();
    setThresholds();    
   
    while(1)
    {  
        /* E-Field MESSUNG */
        
        for (i=0;i<6;i++)
        {
            setFieldSensor(i+1, true);
            delay_ms(1);
            adcval[i] = readADC(0);
            
            /*
            if (adcval[i] <= threshold[i])
            {
                PORTC &= ~(1 << i);
            }
            else
            {
                PORTC |= (1 << i);
            } 
            */  
        } 
        
          
    }
   
}
